@@ -15,10 +15,27 @@
 #' @param maxRound Inner cells contributing to original publishable cells equal to or less than maxRound will be rounded
 #' @param printInc Printing iteration information to console when TRUE  
 #' @param output Possible non-NULL values are \code{"input"}, \code{"inner"} and \code{"publish"}. Then a single data frame is returned.
+#' @param extend0  When `extend0` is set to `TRUE`, the data is automatically extended. 
+#'                 This is relevant when `zeroCandidates = TRUE` (see \code{\link{RoundViaDummy}}).  
+#'        Additionally, `extend0` can be specified as a list, representing the `varGroups` parameter 
+#'        in the \code{\link[SSBtools]{Extend0}} function. 
+#'        Can also be set to `"all"` which means that input codes in hierarchies are considered in addition to those in data.   
 #' @param preAggregate When \code{TRUE}, the data will be aggregated beforehand within the function by the dimensional variables. 
+#' @param aggregatePackage Package used to preAggregate. 
+#'                         Parameter `pkg` to \code{\link[SSBtools]{aggregate_by_pkg}}.
+#' @param aggregateNA Whether to include NAs in the grouping variables while preAggregate. 
+#'                    Parameter `include_na` to \code{\link[SSBtools]{aggregate_by_pkg}}.
+#' @param aggregateBaseOrder Parameter `base_order` to \code{\link[SSBtools]{aggregate_by_pkg}}, used when preAggregate.  
+#'                           The default is set to `FALSE` to avoid unnecessary sorting operations.  
+#'                           When `TRUE`, an attempt is made to return the same result with `data.table` as with base R.
+#'                           This cannot be guaranteed due to potential variations in sorting behavior across different systems.
+#' @param rowGroupsPackage Parameter `pkg` to \code{\link[SSBtools]{RowGroups}}.
+#'               The parameter is input to \code{\link[SSBtools]{Formula2ModelMatrix}} 
+#'               via \code{\link[SSBtools]{ModelMatrix}}. 
 #' @param ... Further parameters sent to \code{RoundViaDummy}  
 #'
-#' @return Output is a four-element list with class attribute "PLSrounded" (to ensure informative printing).
+#' @return Output is a four-element list with class attribute "PLSrounded", 
+#'         which ensures informative printing and enables the use of \code{\link[SSBtools]{FormulaSelection}} on this object.
 #'    \item{inner}{Data frame corresponding to input data with the main dimensional variables and with cell 
 #'                frequencies (original, rounded, difference).}
 #'    \item{publish}{Data frame of publishable data with the main dimensional variables and with cell frequencies 
@@ -38,8 +55,8 @@
 #' 
 #' @encoding UTF8
 #' 
-#' @importFrom SSBtools CharacterDataFrame
-#' @importFrom stats aggregate as.formula delete.response terms
+#' @importFrom SSBtools CharacterDataFrame aggregate_by_pkg NamesFromModelMatrixInput 
+#' @importFrom stats as.formula delete.response terms
 #' @export
 #'
 #' @examples
@@ -55,6 +72,11 @@
 #' print(a$metrics)
 #' print(a$freqTable)
 #' 
+#' # Using FormulaSelection()
+#' FormulaSelection(a$publish, ~eu + year)
+#' FormulaSelection(a, ~eu + year) # same as above
+#' FormulaSelection(a)             # just a$publish
+#' 
 #' # Recalculation of maxdiff, HDutility, meanAbsDiff and rootMeanSquare
 #' max(abs(a$publish[, "difference"]))
 #' HDutility(a$publish[, "original"], a$publish[, "rounded"])
@@ -87,6 +109,12 @@
 #' # Microdata input
 #' PLSroundingInner(rbind(z, z), roundBase = 5, formula = ~geo + eu + year)
 #' 
+#' # Zero perturbed due to both  extend0 = TRUE and zeroCandidates = TRUE 
+#' set.seed(12345)
+#' PLSroundingInner(z[sample.int(5, 12, replace = TRUE), 1:3], 
+#'                  formula = ~geo + eu + year, roundBase = 5, 
+#'                  extend0 = TRUE, zeroCandidates = TRUE, printInc = TRUE)
+#' 
 #' # Parameter avoidHierarchical (see RoundViaDummy and ModelMatrix) 
 #' PLSroundingPublish(z, roundBase = 5, formula = ~geo + eu + year, avoidHierarchical = TRUE)
 #' 
@@ -117,7 +145,13 @@ PLSrounding <- function(data, freqVar = NULL, roundBase = 3, hierarchies = NULL,
                         dimVar = NULL,
                         maxRound = roundBase-1, printInc = nrow(data)>1000, 
                         output = NULL, 
-                        preAggregate = is.null(freqVar), ...) {
+                        extend0 = FALSE,
+                        preAggregate = is.null(freqVar),
+                        aggregatePackage = "base",
+                        aggregateNA = TRUE,
+                        aggregateBaseOrder = FALSE,
+                        rowGroupsPackage = aggregatePackage,
+                        ...) {
   
   
   force(preAggregate)
@@ -132,37 +166,47 @@ PLSrounding <- function(data, freqVar = NULL, roundBase = 3, hierarchies = NULL,
     output <- ""
   }
   
-  if (preAggregate | output == "input") {
+  isExtend0 <- IsExtend0(extend0)
+  
+  if (preAggregate | output == "input" | isExtend0) {
     if (printInc & preAggregate) {
       cat("[preAggregate ", dim(data)[1], "*", dim(data)[2], "->", sep = "")
       flush.console()
     }
-    if (!is.null(hierarchies)) {
-      dVar <- names(hierarchies)
-    } else {
-      if (!is.null(formula)) {
-        dVar <- row.names(attr(delete.response(terms(as.formula(formula))), "factors"))
-      } else {
-        if (!is.null(dimVar)){
-          dVar <- dimVar
-        } else {
+    dVar <- NamesFromModelMatrixInput(hierarchies = hierarchies, formula = formula, dimVar = dimVar)
+    if (!length(dVar)) {
           if (is.null(freqVar)){
             dVar <- names(data)
           } else {
             freqVarName <- names(data[1, freqVar, drop = FALSE])
             dVar <- names(data[1, !(names(data) %in% freqVarName), drop = FALSE])
           }
-        }
-      }
     }
     dVar <- unique(dVar)
     
     if (preAggregate) {
       if (is.null(freqVar)) {
-        data <- aggregate(list(f_Re_qVa_r = data[[dVar[1]]]), data[dVar], length)
+        #data <- aggregate(list(f_Re_qVa_r = data[[dVar[1]]]), data[dVar], length)
+        data <- aggregate_by_pkg(
+          data = data,
+          by = dVar,
+          var = dVar[1],
+          pkg =  aggregatePackage,
+          include_na = aggregateNA,
+          fun = length,
+          base_order = aggregateBaseOrder)
         freqVar <- "f_Re_qVa_r"
+        names(data)[length(dVar) + 1] <- freqVar 
       } else {
-        data <- aggregate(data[freqVar], data[dVar], sum)
+        #data <- aggregate(data[freqVar], data[dVar], sum)
+        data <- aggregate_by_pkg(
+          data = data,
+          by = dVar,
+          var = freqVar,
+          pkg =  aggregatePackage,
+          include_na = aggregateNA,
+          fun = sum,
+          base_order = aggregateBaseOrder)
       }
       if (printInc) {
         cat(dim(data)[1], "*", dim(data)[2], "]\n", sep = "")
@@ -171,6 +215,26 @@ PLSrounding <- function(data, freqVar = NULL, roundBase = 3, hierarchies = NULL,
     }
   }
   
+  if (isExtend0) {
+    if (printInc) {
+      cat("[extend0 ", dim(data)[1], "*", dim(data)[2], "->", sep = "")
+      flush.console()
+    }
+    
+    data <- Extend0fromModelMatrixInput(data = data, 
+                                        freqName = freqVar, 
+                                        hierarchies = hierarchies, 
+                                        formula = formula,
+                                        dimVar = dimVar,
+                                        extend0 = extend0, 
+                                        dVar = dVar, ...)
+    
+    
+    if (printInc) {
+      cat(dim(data)[1], "*", dim(data)[2], "]\n", sep = "")
+      flush.console()
+    }
+  }
   
   if (output == "input"){
     if (!is.null(freqVar)) {
@@ -191,7 +255,8 @@ PLSrounding <- function(data, freqVar = NULL, roundBase = 3, hierarchies = NULL,
   } else {
     z <- RoundViaDummy(data = data, freqVar = freqVar, formula = formula, roundBase = roundBase, hierarchies = hierarchies,
                        dimVar = dimVar,
-                     maxRound = maxRound, printInc = printInc, ...)
+                     maxRound = maxRound, printInc = printInc,
+                     rowGroupsPackage = rowGroupsPackage, ...)
   }
   
   
@@ -225,6 +290,10 @@ PLSrounding <- function(data, freqVar = NULL, roundBase = 3, hierarchies = NULL,
     if (output != "inner"){
       out$publish <- cbind(as.data.frame(z$crossTable[, cNames, drop = FALSE], stringsAsFactors = FALSE), z$yPublish)
       rownames(out$publish) <- NULL
+      startRow <- attr(z$crossTable, "startRow", exact = TRUE)
+      if (!is.null(startRow)) {
+        attr(out$publish, "startRow") <- startRow
+      }
     }
   } else {
     if (output != "publish"){
@@ -299,6 +368,25 @@ print.PLSrounded <- function(x, digits = max(getOption("digits") - 3, 3), ...) {
 }
 
 
+#' FormulaSelection  method for PLSrounded
+#'
+#' @param x PLSrounded object 
+#' @param formula `formula` parameter to \code{\link[SSBtools]{FormulaSelection}}.
+#'        When `NULL` (default), the publish data frame is returned without any limitation.  
+#' @param intercept `intercept` parameter to `FormulaSelection`.
+#' @param logical `logical` parameter to `FormulaSelection`.
+#'
+#' @return Limited version of the publish data frame
+#' @importFrom SSBtools FormulaSelection
+#' @export
+FormulaSelection.PLSrounded <- function(x, formula = NULL, intercept = NA, logical = FALSE) {
+  if (is.null(formula)) {
+    return(x$publish)
+  }
+  SSBtools::FormulaSelection(x$publish, formula, intercept, logical)
+}
+
+
 TabCat <- function(z, b, s, m) {
   x <- rbind(original = table(ToCat(z[, "original"], b, m)), 
              rounded = table(ToCat(z[, "rounded"], b, m)), 
